package com.caimei.module.product.service.impl;

import com.caimei.module.base.entity.bo.PageVo;
import com.caimei.module.base.entity.po.SeconHandProduct;
import com.caimei.module.base.entity.vo.*;
import com.caimei.module.product.dao.ProductModuleDao;
import com.caimei.module.product.service.ProductModuleService;
import com.caimei.module.product.util.ProductUtils;
import com.github.pagehelper.PageHelper;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2020/3/25
 */
@Service
public class ProductModuleServiceImpl implements ProductModuleService {
    @Resource
    private ProductModuleDao productModuleDao;

    /**
     * 获取分类列表
     */
    @Override
    public List<BigtypeVo> getClassify(String domain) {
        List<BigtypeVo> bigTypeList = productModuleDao.getBigType();
        for (BigtypeVo bigType : bigTypeList) {
            List<SmalltypeVo> SmalltypeList = productModuleDao.getSmallType(bigType.getBigTypeID());
            for (SmalltypeVo smalltype : SmalltypeList) {
                List<TinytypeVo> tinytypeList = productModuleDao.getTinytype(smalltype.getSmallTypeID());
                if (!CollectionUtils.isEmpty(tinytypeList)) {
                    for (TinytypeVo tinyType : tinytypeList) {
                        if (StringUtils.isEmpty(tinyType.getIcon())) {
                            tinyType.setIcon(ProductUtils.getImageURL("caiMeiImage", null, 0, domain));
                        }
                    }
                    smalltype.setTinytypeList(tinytypeList);
                }
            }
            bigType.setSmalltypeList(SmalltypeList);
        }
        return bigTypeList;
    }

    /**
     * 根据三级分类获取商品
     *
     * @param tinyTypeID
     * @param sortType
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> getProductsByTinyType(Integer tinyTypeID, Integer sortType, Integer pageNum, Integer pageSize) {
        // sortType (3:价格升序, 4:价格降序, 7:人气, 8:销量)
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        PageHelper.startPage(pageNum, pageSize);
        sortType = null != sortType ? sortType : 8;
        List<ProductVo> productList = productModuleDao.getProductsByTinyType(tinyTypeID, sortType);
        return new PageVo<>(productList);
    }

    /**
     * 获取模块商品列表
     *
     * @param preferredFlag 新品上线(001) 优惠商品(010) 常用商品(100)，三者同时存在111
     * @param userId
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> getPreferred(Integer preferredFlag, Integer userId, Integer pageNum, Integer pageSize) {
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        PageHelper.startPage(pageNum, pageSize);
        List<ProductVo> productList = productModuleDao.getProductsByPreferredFlag(preferredFlag);
        return new PageVo<>(productList);
    }

    /**
     * 根据关键词获取商品
     *
     * @param searchWord
     * @param userId
     * @param pageNum
     * @param pageSize
     */
    @Override
    public PageVo<ProductVo> searchProduct(String searchWord, Integer userId, Integer pageNum, Integer pageSize) {
        pageNum = null != pageNum ? pageNum : 1;
        pageSize = null != pageSize ? pageSize : 10;
        // 登录状态下，查询历史记录是否存在
        if (null != userId) {
            List<SearchHistoryVo> historyList = productModuleDao.getSearchHistoryList(userId);
            //保存并删除大于10条的历史记录
            SearchHistoryVo historyRecord = new SearchHistoryVo();
            historyRecord.setUserId(userId.longValue());
            historyRecord.setSearchDate(new Date());
            historyRecord.setSearchWord(searchWord);
            historyRecord.setDelFlag("0");
            if (historyList != null) {
                Long recordId = productModuleDao.getSearchHistoryIdByWord(searchWord);
                if (recordId != null && recordId > 0L) {
                    historyRecord.setId(recordId);
                    productModuleDao.updateSearchHistoryById(historyRecord);
                } else {
                    productModuleDao.insertSearchHistory(historyRecord);
                }
                if (historyList.size() >= 10) {
                    productModuleDao.deleteSearchHistoryByUserId(userId);
                }
            } else {
                productModuleDao.insertSearchHistory(historyRecord);
            }
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ProductVo> productList = productModuleDao.searchProduct(searchWord);
        return new PageVo<>(productList);
    }


    @Override
    public List<ProductVo> getRecommended(Integer productID) {
        ProductVo product = productModuleDao.findProductById(productID);
        List<ProductVo> productList = null;
        if ("1".equals(product.getRecommendType())) {
            List<Integer> productIDs = productModuleDao.getRecommendIds(productID);
            if (productIDs != null && productIDs.size() > 0) {
                productList = productModuleDao.getRecommendByIds(productIDs);
            }
        } else {
            productList = productModuleDao.getDeFaultRecommend(product.getTinyTypeID());
        }
        return productList;
    }

    @Override
    public AddressVo getAddressInfo(Integer townId){
        return productModuleDao.getAddressInfo(townId);
    }

    @Override
    public void saveSencondHandProduct(SeconHandProduct seconHandProduct) {
        productModuleDao.saveSencondHandProduct(seconHandProduct);
    }

    /**
     * 商品搜索历史记录
     */
    @Override
    public List<SearchHistoryVo> getSearchHistory(Integer userId) {
        return productModuleDao.getSearchHistoryList(userId);
    }

    /**
     * 根据用户ID删除历史记录
     *
     * @param userId
     */
    @Override
    public void deleteAllSearchHistory(Integer userId) {
        productModuleDao.deleteAllSearchHistory(userId);
    }

    /**
     * 商品详情页
     *
     * @param productID
     */
    @Override
    public ProductVo getProductDetails(Integer productID, String domain) {
        ProductVo product = productModuleDao.findProductById(productID);
        String[] split = null;
        //商品标签
        if (product.getTags() != null) {
            String tags = product.getTags();
            tags = tags.replace(" ", ",").replace("、", ",").replace("，", ",");
            if (tags.contains(",")) {
                split = tags.split(",");
                List<String> list = new ArrayList();
                for (String s : split) {
                    if (s != null && !s.equals("")) {
                        list.add(s);
                    }
                }
                split = list.toArray(new String[list.size()]);
            }
        }
        product.setTagsList(split);
        product.setMainImage(ProductUtils.getImageURL("product", product.getMainImage(), 0, domain));
        //商品图片
        List<ProductImageVo> imageList = productModuleDao.findProductImage(product.getProductID());
        for (ProductImageVo image : imageList) {
            String imageURL = ProductUtils.getImageURL("product", image.getImage(), 0, domain);
            image.setImage(imageURL);
        }
        product.setImageList(imageList);
        //商品详情
        ProductDetailVo productDetail = productModuleDao.findProductDetailById(productID);
        product.setProductDetail(productDetail);
        //供应商信息
        ShopVo shop = productModuleDao.findProductShopById(product.getShopID());
        if (shop.getBusinessScope() != null && shop.getBusinessScope().contains("/")) {
            String[] businessScopeArray = shop.getBusinessScope().split("/");
            shop.setBusinessScopeArray(businessScopeArray);
        }
        shop.setLogo(ProductUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
        shop.setBusinessLicenseImage(ProductUtils.getImageURL("shopLogo", shop.getBusinessLicenseImage(), 0, domain));
        product.setShop(shop);
        //价格等级
        String priceGrade = ProductUtils.getPriceGrade(product.getPrice1());
        product.setPrice1Grade(priceGrade);
        return product;
    }

    /**
     * 设置商品展示价格
     *
     * @param product
     * @param userId
     */
    @Override
    public ProductVo getShowPrice(ProductVo product, Integer userId) {
        product.setRetailPrice(new BigDecimal(product.getPrice1()));
        //是否启用阶梯价格
        if (null != userId && product.getLadderPriceFlag().equals("1")) {
            LadderPriceVo ladderPrice = productModuleDao.findLowerLadderPrice(product.getProductID());
            product.setRetailPrice(ladderPrice.getBuyPrice());
        }
        return product;
    }

    /**
     * 获取阶梯价格
     *
     * @param productID
     */
    @Override
    public List<LadderPriceVo> findLadderPrice(Integer productID) {
        return productModuleDao.findLadderPrice(productID);
    }

    /**
     * 获取一条最低阶梯价格
     *
     * @param productID
     */
    @Override
    public LadderPriceVo findLowerLadderPrice(Integer productID) {
        return productModuleDao.findLowerLadderPrice(productID);
    }


}
